
-- #####################################################################################
-- ## Skript zum Vortrag R Services / Machine Learning Services mit SQL Server        ##     
-- #####################################################################################
-- ## Dieses Skript legt die Projektdatenbank an. Es wird durch weitere Skripte       ##
-- ## ergnzt, die in ihrer durchnummerierten Reihenfolge abzuarbeiten sind. Die      ##
-- ## Skripte passen zu den Aufgabenstellungen aus der zugehrigen Powerpoint-        ##
-- ## Prsentation vom selben Autor.                                                  ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Markus Delhofen, arelium GmbH        markus.delhofen@arelium.de                 ##
-- #####################################################################################


-- Projekt- / Demodatenbank auswhlen
USE [R_Services_Demo]
GO


/*  Ausgabe der Standardabweichung als Resultset (Fehler)
 *  -----------------------------------------------------
 *  Parameter @output_data_1_name legt den Namen 
 *  des Dataframes im R Script fest, der zurckgegeben wird.
 *
 */ 
DECLARE @rscript nvarchar(max)

 SET @rscript = N'
	x <- c(-2, -1, -0.8, -0.3, 0, 0.5, 0.6, 0.7, 1, 1.2);
	abweichung <- sd(x);   #Fehler!!
' 

 EXEC sp_execute_external_script  
      @language = N'R'  
     ,@script = @rscript
     ,@output_data_1_name = N'abweichung'  
GO
--> Problem: Die Ausgabe muss ein Dataframe sein.
 










/*  Ausgabe der Standardabweichung  als Resultset (korrekt)
 *  -------------------------------------------------------
 *  Parameter @output_data_1_name legt den Namen 
 *  des Dataframes im R Script fest, der zurckgegeben wird.
 *
 *  Umwandlung des skaleren Wertes mittels data.frame()
 *
 */ 
DECLARE @rscript nvarchar(max)

 SET @rscript = N'
	x <- c(-2, -1, -0.8, -0.3, 0, 0.5, 0.6, 0.7, 1, 1.2);
	abweichung <- sd(x);

	# Konvertierung in ein DataFrame:
	result <- data.frame(abweichung);
' 

 EXEC sp_execute_external_script  
      @language = N'R'  
     ,@script = @rscript
     ,@output_data_1_name = N'result'  
GO
--> Die Spalte hat keine berschrift und einen implizieten Typ.
-- 110





/*  Ausgabe der Standardabweichung  als Dataframe (korrket)
 *  -------------------------------------------------------
 *  Format des Datasets wird ber WITH RESULT SETS festgelegt.
 *
 */ 
DECLARE @rscript nvarchar(max)

 SET @rscript = N'
	x <- c(-2, -1, -0.8, -0.3, 0, 0.5, 0.6, 0.7, 1, 1.2);
	abweichung <- sd(x);

	# Konvertierung in ein DataFrame:
	result <- data.frame(abweichung);
' 

 EXEC sp_execute_external_script  
      @language = N'R'  
     ,@script = @rscript
     ,@output_data_1_name = N'result'  
    WITH RESULT SETS ((
		abweichung     float
	));  
GO
--> 140






/*  Erweiterung des Dataframe um Maximum (korrekt)
 *  -------------------------------------------------------
 *  Format des Datasets wird ber WITH RESULT SETS festgelegt.
 *  zwei Werte in einem Vector verknpfen
 *
 */ 
DECLARE @rscript nvarchar(max)

 SET @rscript = N'
	x <- c(-2, -1, -0.8, -0.3, 0, 0.5, 0.6, 0.7, 1, 1.2);
	abweichung <- sd(x);
    maximum <- max(x);

	werte <- c(abweichung, maximum)

	# Konvertierung in ein DataFrame:
	result <- data.frame(werte);
' 

 EXEC sp_execute_external_script  
      @language = N'R'  
     ,@script = @rscript
     ,@output_data_1_name = N'result'  
    WITH RESULT SETS ((
	     [Wert]    decimal(10,2)
	));  
GO
--> 180












/*  Erweiterung des Dataframe um Maximum (korrekt)
 *  -------------------------------------------------------
 *  Format des Datasets wird ber WITH RESULT SETS festgelegt.
 *  zwei Werte in einem Vector verknpfen mit Beschreibung
 *
 */ 
DECLARE @rscript nvarchar(max)

 SET @rscript = N'
	x <- c(-2, -1, -0.8, -0.3, 0, 0.5, 0.6, 0.7, 1, 1.2);
	abweichung <- sd(x);
    maximum <- max(x);

	namen <- c(''Abweichung'', "Maximum")
	werte <- c(abweichung, maximum)

	# Konvertierung in ein DataFrame:
	result <- data.frame(namen, werte);
' 

 EXEC sp_execute_external_script  
      @language = N'R'  
     ,@script = @rscript
     ,@output_data_1_name = N'result'  
    WITH RESULT SETS ((
	     [Beschreibung]  nvarchar(200),
		 [wert]          float
	));  

GO
--> 220









/*  Erweiterung des Dataframe um Maximum (korrekt)
 *  -------------------------------------------------------
 *  Format des Datasets wird ber WITH RESULT SETS festgelegt.
 *  zwei Werte in zwei Spalten
 *
 */ 
DECLARE @rscript nvarchar(max)

 SET @rscript = N'
	x <- c(-2, -1, -0.8, -0.3, 0, 0.5, 0.6, 0.7, 1, 1.2);
	abweichung <- sd(x);
    maximum <- max(x);

	# Konvertierung in ein DataFrame:
	result <- data.frame(abweichung, maximum);
' 

 EXEC sp_execute_external_script  
      @language = N'R'  
     ,@script = @rscript
     ,@output_data_1_name = N'result'  
    WITH RESULT SETS ((
	     [Abweichung]  decimal(9,5),
		 [Maximum]     decimal(5,2)
	));  

GO
